<?php
declare(strict_types=1);

namespace Kibra;

class IblockList
{
    private function orderResult()
    {
        if(count($this->order)>0) {
            $sortKey = array_key_first($this->order);
            if(isset($this->resultArray[0][$sortKey])) {
                $sortMethod = $this->order[$sortKey];
                switch ($sortMethod) {
                    case 'ASC':
                        usort($this->resultArray, function($a, $b) use ($sortKey) {return strcmp($a[$sortKey], $b[$sortKey]);});
                        break;
                    case 'DESC':
                        usort($this->resultArray, function($a, $b) use ($sortKey) {return strcmp($b[$sortKey], $a[$sortKey]);});
                        break;
                };
            } else {
                $this->error = [
                    'error' => 'Wrong sort key',
                    'error_description' => 'The sort key is not present in the selected keys. Sorting is impossible.',
                ];
            }
        }
    }

    private function compileResult(array $data)
    {
        foreach($data as $dataItem) {
            $this->resultArray = array_merge($this->resultArray, $dataItem);
            $this->nextID = $this->resultArray[count($this->resultArray)-1]['ID'];
        }
    }

    private function batch(float $batchCount):bool
    {
        $arBatch = [];
        for($batchItem = 0; $batchItem < $batchCount; $batchItem++) {
            if ($batchItem == 0) {
                $filter = array_merge($this->filter, ["<ID" => $this->nextID]);
            } else {
                $filter = array_merge($this->filter, ['<ID' => '$result[' . strval($batchItem-1) . '][49][ID]']);
            }
            $arParams = array_merge($this->params, [
                'start' => -1,
                'IBLOCK_TYPE_ID' => $this->iblockTypeId,
                'ELEMENT_ORDER' => ['ID' => 'DESC'],
                'FILTER' => $filter,
            ]);
            $arBatch[]=[
                'method' => $this->method,
                'params' => $arParams,
            ];
            $this->countOfBatches--;
        };
        if(count($arBatch)>0) {
            $result = $this->crest::callBatch($arBatch);
            if ($result['result']['result_error']) {
                $this->error = $result['result']['result_error'];
                return false;
            } else {
                $this->compileResult($result['result']['result']);
                return true;
            }
        };
    }

    public function getList():array
    {
        if (count($this->error)>0) {
            return ['result' => $this->resultArray, 'total' => $this->total, 'error' => $this->error];
        } else if ($this->resultPresent) {
            $this->orderResult();
            return ['result' => $this->resultArray, 'total' => $this->total, 'error' => $this->error];
        } else {
            for($page = 0; $page < $this->countOfPages; $page++){
                if (!$this->batch(50)) return ['result' => $this->resultArray, 'total' => $this->total, 'error' => $this->error];
            }
            if (!($this->batch($this->countOfBatches))) return ['result' => $this->resultArray, 'total' => $this->total, 'error' => $this->error];   
            $this->orderResult();     
            return ['result' => $this->resultArray, 'total' => $this->total, 'error' => $this->error];
        }        
    }

    private function init()
    {
        $arParams = array_merge($this->params, [
            'IBLOCK_TYPE_ID' => $this->iblockTypeId,
            'ELEMENT_ORDER' => ['ID' => 'DESC'],
            'FILTER' => $this->filter,
        ]);
        $result = $this->crest::call(
            $this->method, 
            $arParams  
        );
        if (isset($result['error'])) {
            $this->error = [
                'error' => $result['error'],
                'error_description' => isset($result['error_description']) ? $result['error_description'] : '',
                'error_information' => isset($result['error_information']) ? $result['error_information'] : '',
            ];
            return false;
        } else {
            $this->total = isset($result['total']) ? $result['total'] : 0;
            if ($this->total<=50) {
                $this->resultArray = $result['result'];
                $this->resultPresent = true;
            } else {
                $maxID = $result['result'][0]['ID'];
                $this->nextID = $maxID + 1;
                $this->countOfBatches = ceil($this->total/50);
                $this->countOfPages = intdiv($this->total, 2500);
            }            
        }   
    }

    public function __construct(string $crest, array $params=[]) 
    {
        $this->crest = $crest;

        $this->resultPresent = false;
        $this->resultArray = [];
        $this->total = 0;
        $this->error = [];
        $this->nextID = 0;

        $this->params = [];

        $this->method = 'lists.element.get';
        $this->iblockTypeId = isset($params['iblockTypeId']) ? $params['iblockTypeId'] : 'lists';
        $this->iblockId = isset($params['iblockId']) ? $params['iblockId'] : '';
        $this->elementId = isset($params['elementId']) ? $params['elementId'] : '';

        $this->iblockCode = isset($params['iblockCode']) ? $params['iblockCode'] : '';
        $this->elementCode = isset($params['elementCode']) ? $params['elementCode'] : '';
        $this->socnetGroupId = isset($params['socnetGroupId']) ? $params['socnetGroupId'] : '';

        if ($this->socnetGroupId) {
            $this->params['SOCNET_GROUP_ID'] = $this->socnetGroupId;
        };
        if ($this->iblockId) {
            $this->params['IBLOCK_ID'] = $this->iblockId;
        };
        if ($this->iblockCode) {
            $this->params['IBLOCK_CODE'] = $this->iblockCode;
        };
        if ($this->elementId) {
            $this->params['ELEMENT_ID'] = $this->elementId;
        };
        if ($this->elementCode) {
            $this->params['ELEMENT_CODE'] = $this->elementCode;
        };

        $this->filter = isset($params['filter']) ? $params['filter'] : [];
        $this->order = isset($params['order']) ? $params['order'] : [];
        $this->init();
     }
}